CumulativeROHistogram32

Struct CumulativeROHistogram32 

pub struct CumulativeROHistogram32 { /* private fields */ }
A read-only, cumulative histogram for fast quantile queries.

This is a variant of the SparseHistogram with cumulative counts (starting from the first bucket) for each bucket that is present.

Stores only non-zero buckets in columnar form, like SparseHistogram, but with cumulative counts: count[i] equals the total number of observations in buckets 0..=i (i.e., a running prefix sum). The last element of count equals the total observation count.

CumulativeROHistogram is intended to be read-only—i.e. it shouldn’t accept updates for new observations, because such operations would be expensive given counts are cumulative. On the other hand, querying percentiles is cheaper than standard or sparse histograms without cumulative counts, which can be performed with binary search. Additional methods to provide the percentile range each or all bucket(s) represent are implmented to facilitate analytics based on such histograms.

Implementations

Source

impl CumulativeROHistogram32

pub fn from_parts( config: Config, index: Vec<u32>, count: Vec<u32>, ) -> Result<Self, Error>

Creates a cumulative histogram from its raw parts.

The count vector must contain cumulative counts (a running prefix sum of individual bucket counts).

Returns an error if:

  • index and count have different lengths
  • any index is out of range for the config
  • the indices are not in strictly ascending order
  • the counts are not strictly non-decreasing
  • any count is zero
pub fn into_parts(self) -> (Config, Vec<u32>, Vec<u32>)

Consumes the histogram, returning the config, index, and cumulative count vectors.

pub fn config(&self) -> Config

Returns the bucket configuration.

pub fn index(&self) -> &[u32]

Returns a slice of the non-zero bucket indices.

pub fn count(&self) -> &[u32]

Returns a slice of the cumulative bucket counts.

pub fn total_count(&self) -> u64

Returns the total number of observations across all buckets.

pub fn len(&self) -> usize

Returns the number of non-zero buckets.

pub fn is_empty(&self) -> bool

Returns true if the histogram contains no observations.

pub fn bucket_quantile_range(&self, bucket_idx: usize) -> Option<(f64, f64)>

Returns the quantile range (lower, upper) for the bucket at position bucket_idx in the sparse representation.

  • lower is the fraction of observations strictly before this bucket (in [0.0, 1.0]).
  • upper is the fraction of observations at or before this bucket (in [0.0, 1.0]).

Returns None if the histogram is empty or bucket_idx is out of range.

pub fn iter_with_quantiles(&self) -> QuantileRangeIter32<'_>

Returns an iterator yielding (Bucket, lower_quantile, upper_quantile) for each non-zero bucket.

Each Bucket contains the individual (non-cumulative) count. The quantile range (lower, upper) indicates the fraction of total observations before and up to this bucket.

pub fn iter(&self) -> CumulativeIter32<'_>

Returns an iterator across the non-zero histogram buckets.

Each Bucket contains the individual (non-cumulative) count for that bucket.

pub fn quantiles( &self, quantiles: &[f64], ) -> Result<Option<QuantilesResult>, Error>

Compute quantiles for the given values.

pub fn quantile(&self, quantile: f64) -> Result<Option<QuantilesResult>, Error>

Compute a single quantile.

pub fn as_ref(&self) -> CumulativeROHistogram32Ref<'_>

Returns a borrowed view over this histogram’s storage.